@@ -639,6 +639,82 @@ describe('applyToDefaults()', () => {
         expect(merged).to.equal({ a: { b: 4 }, c: { g: { r: { h: 8 } } } });
     });
 
+    it('shallow copies the nested keys (non-object)', () => {
+
+        const defaults = {
+            // All falsy values:
+            _undefined: {
+                a: 1
+            },
+            _null: {
+                a: 2
+            },
+            _false: {
+                a: 3
+            },
+            _emptyString: {
+                a: 4
+            },
+            _zero: {
+                a: 5
+            },
+            _NaN: {
+                a: 6
+            },
+            // Other non-object values:
+            _string: {
+                a: 7
+            },
+            _number: {
+                a: 8
+            },
+            _true: {
+                a: 9
+            },
+            _function: {
+                a: 10
+            }
+        };
+
+        const source = {
+            _undefined: undefined,
+            _null: null,
+            _false: false,
+            _emptyString: '',
+            _zero: 0,
+            _NaN: NaN,
+            _string: 'foo',
+            _number: 42,
+            _true: true,
+            _function: () => {}
+        };
+
+        const merged = Hoek.applyToDefaults(defaults, source, { shallow: [
+            '_undefined.a',
+            '_null.a',
+            '_false.a',
+            '_emptyString.a',
+            '_zero.a',
+            '_NaN.a',
+            '_string.a',
+            '_number.a',
+            '_true.a',
+            '_function.a'
+        ] });
+        expect(merged).to.equal({
+            _undefined: { a: 1 },
+            _null: { a: 2 },
+            _false: false,
+            _emptyString: '',
+            _zero: 0,
+            _NaN: NaN,
+            _string: 'foo',
+            _number: 42,
+            _true: true,
+            _function: source._function
+        });
+    });
+
     it('shallow copies the listed keys in the defaults', () => {
 
         const defaults = {
@@ -770,6 +846,16 @@ describe('applyToDefaults()', () => {
         expect(merged.a).to.shallow.equal(source.a);
         expect(merged.a).to.not.equal(defaults.a);
     });
+
+    it('should respect nullOverride when shallow is used', () => {
+
+        const defaults = { host: 'localhost', port: 8000 };
+        const source = { host: null, port: 8080 };
+
+        const result = Hoek.applyToDefaults(defaults, source, { nullOverride: true, shallow: [] });
+        expect(result.host).to.equal(null);
+        expect(result.port).to.equal(8080);
+    });
 });
 
 describe('deepEqual()', () => {
@@ -1983,6 +2069,30 @@ describe('reachTemplate()', () => {
 
         expect(Hoek.reachTemplate(obj, template, '/')).to.equal('4:9:1::');
     });
+
+    it('isn\'t prone to ReDoS given an adversarial template', () => {
+
+        const sizes = [0, 1, 2, 3, 4]; // Should be evenly-spaced
+        const times = [];
+        const diffs = [];
+
+        for (const size of sizes) {
+            const start = Date.now();
+            Hoek.reachTemplate({}, '{'.repeat(size * 10000));
+            times.push(Date.now() - start);
+        }
+
+        for (let i = 1; i < times.length; ++i) {
+            diffs.push(times[i] - times[i - 1]);
+        }
+
+        // Under ReDoS, as the size of the input increases the timing accelerates upwards,
+        // i.e. each timing diff would be greater than the last.
+
+        const diffsMonotonic = diffs[0] < diffs[1] && diffs[1] < diffs[2] && diffs[2] < diffs[3];
+
+        expect(diffsMonotonic, 'Timing diffs monotonic').to.be.false();
+    });
 });
 
 describe('assert()', () => {
@@ -2031,7 +2141,7 @@ describe('assert()', () => {
 
         expect(() => {
 
-            Hoek.assert(false, 'This', 'is', new Error('spinal'), new Error('tap'));
+            Hoek.assert(false, new Error('This'), 'is', 'spinal', new Error('tap'));
         }).to.throw('This is spinal tap');
     });
 
@@ -2231,6 +2341,87 @@ describe('wait()', () => {
         expect(timeout.after).to.be.undefined();
     });
 
+    it('delays for timeout ms as bigint', async () => {
+
+        const timeout = {};
+        setTimeout(() => (timeout.before = true), 10);
+        const wait = Hoek.wait(10n);
+        setTimeout(() => (timeout.after = true), 10);
+
+        await wait;
+
+        expect(timeout.before).to.be.true();
+        expect(timeout.after).to.be.undefined();
+    });
+
+    it('handles timeouts >= 2^31', async () => {
+
+        const flow = [];
+        let no = 0;
+
+        const fakeTimeout = function (cb, time) {
+
+            const timer = ++no;
+
+            flow.push(`CALL(${timer}): ${time}`);
+            setImmediate(() => {
+
+                flow.push(`PRE(${timer})`);
+                cb();
+                flow.push(`POST(${timer})`);
+            });
+        };
+
+        await Hoek.wait(2 ** 31, null, { setTimeout: fakeTimeout });
+        flow.push('DONE1');
+        await Hoek.wait(2 ** 32 + 2 ** 30, null, { setTimeout: fakeTimeout });
+        flow.push('DONE2');
+
+        expect(flow).to.equal([
+            'CALL(1): 2147483647',
+            'PRE(1)',
+            'CALL(2): 1',
+            'POST(1)',
+            'PRE(2)',
+            'POST(2)',
+            'DONE1',
+            'CALL(3): 2147483647',
+            'PRE(3)',
+            'CALL(4): 2147483647',
+            'POST(3)',
+            'PRE(4)',
+            'CALL(5): 1073741826',
+            'POST(4)',
+            'PRE(5)',
+            'POST(5)',
+            'DONE2'
+        ]);
+    });
+
+    it('returns never resolving promise when timeout >= Number.MAX_SAFE_INTEGER', async () => {
+
+        let calls = 0;
+        const fakeTimeout = function (cb) {
+
+            ++calls;
+            process.nextTick(cb);
+        };
+
+        await Hoek.wait(2 ** 31 - 1, null, { setTimeout: fakeTimeout });
+        expect(calls).to.equal(1);
+
+        const waited = Symbol('waited');
+
+        const result = await Promise.race([
+            Hoek.wait(1, waited),
+            Hoek.wait(Number.MAX_SAFE_INTEGER, null, { setTimeout: fakeTimeout }),
+            Hoek.wait(Infinity, null, { setTimeout: fakeTimeout })
+        ]);
+
+        expect(result).to.be.equal(waited);
+        expect(calls).to.equal(1);
+    });
+
     it('handles a return value', async () => {
 
         const uniqueValue = {};
@@ -2270,7 +2461,6 @@ describe('wait()', () => {
 
         await expect(() => Hoek.wait({})).to.throw();
         await expect(() => Hoek.wait(Symbol('hi'))).to.throw();
-        await expect(() => Hoek.wait(BigInt(10))).to.throw();
     });
 });
 
